@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.c,v 1.9 2006/01/02 14:26:40 paolo Exp $
+ * $Id: gedit-window.c,v 1.15 2006/01/20 09:24:04 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -165,6 +165,8 @@ gedit_window_destroy (GtkObject *object)
 		g_object_unref (window->priv->recent_view_uim);
 		window->priv->recent_view_uim = NULL;
 	}
+	
+	gedit_plugins_engine_garbage_collect();
 
 	GTK_OBJECT_CLASS (gedit_window_parent_class)->destroy (object);
 }
@@ -183,7 +185,7 @@ gedit_window_window_state_event (GtkWidget           *widget,
 		gboolean show;
 
 		show = !(event->new_window_state &
-		         (GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN));
+			(GDK_WINDOW_STATE_MAXIMIZED | GDK_WINDOW_STATE_FULLSCREEN));
 
 		_gedit_statusbar_set_has_resize_grip (GEDIT_STATUSBAR (window->priv->statusbar),
 						      show);
@@ -233,6 +235,13 @@ gedit_window_key_press_event (GtkWidget   *widget,
 		return GTK_WIDGET_CLASS (gedit_window_parent_class)->key_press_event (widget, event);
 }
 
+static void
+gedit_window_tab_removed (GeditWindow         *window,
+			      GeditTab *tab) 
+{
+	gedit_plugins_engine_garbage_collect();
+}
+
 static void
 gedit_window_class_init (GeditWindowClass *klass)
 {
@@ -240,6 +249,8 @@ gedit_window_class_init (GeditWindowClass *klass)
 	GtkObjectClass *gobject_class = GTK_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+	klass->tab_removed = gedit_window_tab_removed;
+
 	object_class->finalize = gedit_window_finalize;
 	object_class->get_property = gedit_window_get_property;
 
@@ -313,7 +324,7 @@ gedit_window_class_init (GeditWindowClass *klass)
 
 static void
 menu_item_select_cb (GtkMenuItem *proxy,
-                     GeditWindow *window)
+		     GeditWindow *window)
 {
 	GtkAction *action;
 	char *message;
@@ -839,7 +850,7 @@ recent_tooltip_func_gtk (GtkTooltips   *tooltips,
 
 	trunc_uri = gedit_utils_str_middle_truncate (uri_for_display,
 						     TIP_MAX_URI_LEN);
-        g_free (uri_for_display);
+	g_free (uri_for_display);
 
 	/* Translators: %s is a URI */
 	tip = g_strdup_printf (_("Open '%s'"), trunc_uri);
@@ -864,7 +875,7 @@ recent_tooltip_func_uim (EggRecentItem *item,
 
 	trunc_uri = gedit_utils_str_middle_truncate (uri_for_display,
 						     TIP_MAX_URI_LEN);
-        g_free (uri_for_display);
+	g_free (uri_for_display);
 
 	/* Translators: %s is a URI */
 	tip = g_strdup_printf (_("Open '%s'"), trunc_uri);
@@ -917,7 +928,7 @@ set_non_homogeneus (GtkWidget *widget, gpointer data)
 {
 	gtk_tool_item_set_homogeneous (GTK_TOOL_ITEM (widget), FALSE);
 }
-                                             
+
 static void
 create_menu_bar_and_toolbar (GeditWindow *window, 
 			     GtkWidget   *main_box)
@@ -960,6 +971,10 @@ create_menu_bar_and_toolbar (GeditWindow *window,
 				      gedit_menu_entries,
 				      G_N_ELEMENTS (gedit_menu_entries),
 				      window);
+	gtk_action_group_add_toggle_actions (action_group,
+					     gedit_toggle_menu_entries,
+					     G_N_ELEMENTS (gedit_toggle_menu_entries),
+					     window);
 
 	gtk_ui_manager_insert_action_group (manager, action_group, 0);
 	g_object_unref (action_group);
@@ -991,10 +1006,6 @@ create_menu_bar_and_toolbar (GeditWindow *window,
 	window->priv->quit_action_group = action_group;
 
 	/* now load the UI definition */
-	if (gtk_ui_manager_add_ui_from_file (manager, "gedit-ui.xml", NULL)) // REMOVE ME... just to allow running without install
-		;
-	else
-
 	gtk_ui_manager_add_ui_from_file (manager, GEDIT_UI_DIR "gedit-ui.xml", &error);
 	if (error != NULL)
 	{
@@ -1782,7 +1793,15 @@ drag_data_received_cb (GtkWidget        *widget,
 	uris = g_uri_list_extract_uris ((gchar *) selection_data->data);
 
 	for (i = 0; uris[i] != NULL; i++)
-		uri_list = g_slist_prepend (uri_list, g_strdup (uris[i]));
+	{
+		gchar *uri;
+		
+		uri = gedit_utils_make_canonical_uri_from_shell_arg (uris[i]);
+		
+		/* Silently ignore malformed URI/filename */
+		if (uri != NULL)
+			uri_list = g_slist_prepend (uri_list, uri);	
+	}
 
 	g_strfreev (uris);
 
@@ -2384,6 +2403,33 @@ notebook_popup_menu (GtkNotebook *notebook,
 	return FALSE;
 }
 
+static void
+hpaned_restore_position (GtkWidget   *widget,
+			 GeditWindow *window)
+{
+	gint pos;
+
+	pos = MAX (100, gedit_prefs_manager_get_side_panel_size ());
+	gtk_paned_set_position (GTK_PANED (window->priv->hpaned), pos);
+
+	/* run this only once */
+	g_signal_handlers_disconnect_by_func (widget, hpaned_restore_position, window);
+}
+
+static void
+vpaned_restore_position (GtkWidget   *widget,
+			 GeditWindow *window)
+{
+	gint pos;
+
+	pos = widget->allocation.height -
+	      MAX (50, gedit_prefs_manager_get_bottom_panel_size ());
+	gtk_paned_set_position (GTK_PANED (window->priv->vpaned), pos);
+
+	/* run this only once */
+	g_signal_handlers_disconnect_by_func (widget, vpaned_restore_position, window);
+}
+
 static void
 side_panel_size_allocate (GtkWidget     *widget,
 			  GtkAllocation *allocation,
@@ -2402,8 +2448,6 @@ side_panel_hide (GtkWidget   *panel,
 static void
 create_side_panel (GeditWindow *window)
 {
-	GtkAction *action;
-	gboolean visible;
 	GtkWidget *documents_panel;
 
 	gedit_debug (DEBUG_WINDOW);
@@ -2434,17 +2478,6 @@ create_side_panel (GeditWindow *window)
 					      documents_panel,
 					      "Documents",
 					      GTK_STOCK_FILE);
-
-	visible = gedit_prefs_manager_get_side_pane_visible ();
-
-	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					      "ViewSidePane");		
-
-	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
-
-	if (visible)
-		gtk_widget_show (window->priv->side_panel);
 }
 
 static void
@@ -2462,49 +2495,123 @@ bottom_panel_hide (GtkWidget   *panel,
 	_gedit_window_set_bottom_panel_visible (window, FALSE);
 }
 
+static void
+bottom_panel_item_removed (GeditPanel  *panel,
+			   GtkWidget   *item,
+			   GeditWindow *window)
+{
+	if (gedit_panel_get_n_items (panel) == 0)
+	{
+		GtkAction *action;
+
+		_gedit_window_set_bottom_panel_visible (window, FALSE);
+
+		action = gtk_action_group_get_action (window->priv->action_group,
+						      "ViewBottomPanel");
+		gtk_action_set_sensitive (action, FALSE);
+	}
+}
+
+static void
+bottom_panel_item_added (GeditPanel  *panel,
+			 GtkWidget   *item,
+			 GeditWindow *window)
+{
+	/* if it's the first item added, set the menu item
+	 * sensitive and if needed show the panel */
+	if (gedit_panel_get_n_items (panel) == 1)
+	{
+		GtkAction *action;
+		gboolean show;
+
+		action = gtk_action_group_get_action (window->priv->action_group,
+						      "ViewBottomPanel");
+		gtk_action_set_sensitive (action, TRUE);
+
+		show = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+		_gedit_window_set_bottom_panel_visible (window, show);
+	}
+}
+
 static void
 create_bottom_panel (GeditWindow *window) 
 {
-	GtkAction *action;
-	gboolean visible;
-
 	gedit_debug (DEBUG_WINDOW);
 
 	window->priv->bottom_panel = gedit_panel_new (GTK_ORIENTATION_HORIZONTAL);
-	
+
 	gtk_paned_pack2 (GTK_PANED (window->priv->vpaned), 
 			 window->priv->bottom_panel, 
-			 FALSE, 
+			 FALSE,
 			 FALSE);
 
 	g_signal_connect (window->priv->bottom_panel,
 			  "size_allocate",
 			  G_CALLBACK (bottom_panel_size_allocate),
 			  window);
-
   	g_signal_connect (window->priv->bottom_panel,
 			  "hide",
 			  G_CALLBACK (bottom_panel_hide),
 			  window);
 
 	gtk_paned_set_position (GTK_PANED (window->priv->vpaned),
-				gedit_prefs_manager_get_bottom_panel_size ()); // CHECK set a min height as we do for the sidepane?
+				MAX (50, gedit_prefs_manager_get_bottom_panel_size ()));
+}
 
-	visible = gedit_prefs_manager_get_bottom_panel_visible();
+static void
+init_panes_visibility (GeditWindow *window)
+{
+	GtkAction *action;
+	gboolean visible;
+
+	gedit_debug (DEBUG_WINDOW);
+
+	/* side pane */
+	visible = gedit_prefs_manager_get_side_pane_visible ();
 
 	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
-					      "ViewBottomPanel");
+					      "ViewSidePane");		
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
-	if(visible) 
-  		gtk_widget_show_all (window->priv->bottom_panel);
+	if (visible)
+		gtk_widget_show (window->priv->side_panel);
+
+	/* bottom pane, it can be empty */
+	action = gtk_action_group_get_action (window->priv->action_group,
+					      "ViewBottomPanel");
+
+	if (gedit_panel_get_n_items (GEDIT_PANEL (window->priv->bottom_panel)) > 0)
+	{
+		visible = gedit_prefs_manager_get_bottom_panel_visible();
+
+		if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
+			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
+
+		if(visible)
+  			gtk_widget_show (window->priv->bottom_panel);
+	}
+	else
+	{
+		gtk_action_set_sensitive (action, FALSE);
+	}
+
+	/* start track sensitivity after the initial state is set */
+  	g_signal_connect (window->priv->bottom_panel,
+			  "item_removed",
+			  G_CALLBACK (bottom_panel_item_removed),
+			  window);
+  	g_signal_connect (window->priv->bottom_panel,
+			  "item_added",
+			  G_CALLBACK (bottom_panel_item_added),
+			  window);
 }
 
 static void
 gedit_window_init (GeditWindow *window)
 {
+	static gboolean is_first = TRUE;
 	GtkWidget *main_box;
 
 	gedit_debug (DEBUG_WINDOW);
@@ -2523,11 +2630,9 @@ gedit_window_init (GeditWindow *window)
 	gtk_widget_show (main_box);
 
 	/* Add menu bar and toolbar bar */
-	gedit_debug_message (DEBUG_WINDOW, "Add menu bar and toolbar bar");
 	create_menu_bar_and_toolbar (window, main_box);
 
 	/* Add status bar */
-	gedit_debug_message (DEBUG_WINDOW, "Add statusbar");	
 	create_statusbar (window, main_box);
 
 	/* Add the main area */
@@ -2538,17 +2643,12 @@ gedit_window_init (GeditWindow *window)
   			    TRUE, 
   			    TRUE, 
   			    0);
-	gtk_widget_show (window->priv->hpaned);
 
-	gedit_debug_message (DEBUG_WINDOW, "Create side panel");		
-  	create_side_panel (window);
-  	  	
 	window->priv->vpaned = gtk_vpaned_new ();
   	gtk_paned_pack2 (GTK_PANED (window->priv->hpaned), 
   			 window->priv->vpaned, 
   			 TRUE, 
   			 FALSE);
-  	gtk_widget_show (window->priv->vpaned);
   	
 	gedit_debug_message (DEBUG_WINDOW, "Create gedit notebook");
 	window->priv->notebook = gedit_notebook_new ();
@@ -2558,13 +2658,32 @@ gedit_window_init (GeditWindow *window)
   			 TRUE);
   	gtk_widget_show (window->priv->notebook);  			 
 
-	gedit_debug_message (DEBUG_WINDOW, "Create bottom panel");
+	/* side and bottom panels */
+  	create_side_panel (window);
 	create_bottom_panel (window);
-	
-	/* Set visibility of panels */
-	// TODO
 
-	gedit_debug_message (DEBUG_WINDOW, "Connect signals");
+	/* restore paned positions as soon as we know the panes allocation.
+	 * This needs to be done only for the first window, the successive
+	 * windows are created by cloning the first one */
+	if (is_first)
+	{
+		is_first = FALSE;
+
+		window->priv->side_panel_size = gedit_prefs_manager_get_side_panel_size ();
+		window->priv->bottom_panel_size = gedit_prefs_manager_get_bottom_panel_size ();
+		g_signal_connect_after (window->priv->hpaned,
+					"map",
+					G_CALLBACK (hpaned_restore_position),
+					window);
+		g_signal_connect_after (window->priv->vpaned,
+					"map",
+					G_CALLBACK (vpaned_restore_position),
+					window);
+	}
+
+	gtk_widget_show (window->priv->hpaned);
+	gtk_widget_show (window->priv->vpaned);
+
 	/* Drag and drop support */
 	gtk_drag_dest_set (GTK_WIDGET (window),
 			   GTK_DEST_DEFAULT_MOTION |
@@ -2608,31 +2727,35 @@ gedit_window_init (GeditWindow *window)
 			  G_CALLBACK (notebook_popup_menu),
 			  window);
 
-	/* connect instead pf override, so that we can
+	/* connect instead of override, so that we can
 	 * share the cb code with the view */
 	g_signal_connect (window,
 			  "drag_data_received",
 	                  G_CALLBACK (drag_data_received_cb), 
 	                  NULL);
-	                  
-	gedit_debug_message (DEBUG_WINDOW, "Update plugins ui");	                  
-        gedit_plugins_engine_update_plugins_ui (window, TRUE);
-        
-       	gedit_debug_message (DEBUG_WINDOW, "END");	                  
+
+	gedit_debug_message (DEBUG_WINDOW, "Update plugins ui");
+	gedit_plugins_engine_update_plugins_ui (window, TRUE);
+
+	/* set visibility of panes.
+	 * This needs to be done after plugins activatation */
+	init_panes_visibility (window);
+
+	gedit_debug_message (DEBUG_WINDOW, "END");
 }
 
 GeditView *
 gedit_window_get_active_view (GeditWindow *window)
 {
 	GeditView *view;
-	
+
 	g_return_val_if_fail (GEDIT_IS_WINDOW (window), NULL);
-	
+
 	if (window->priv->active_tab == NULL)
 		return NULL;
-		
+
 	view = gedit_tab_get_view (GEDIT_TAB (window->priv->active_tab));
-	
+
 	return view;
 }
 
@@ -2640,13 +2763,13 @@ GeditDocument *
 gedit_window_get_active_document (GeditWindow *window)
 {
 	GeditView *view;
-	
+
 	g_return_val_if_fail (GEDIT_IS_WINDOW (window), NULL);
-	
+
 	view = gedit_window_get_active_view (window);
 	if (view == NULL)
 		return NULL;
-	
+
 	return GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 }
 
@@ -2673,7 +2796,7 @@ gedit_window_create_tab (GeditWindow *window,
 				tab,
 				-1,
 				jump_to);
-				
+
 	return tab;
 }
 
@@ -2689,14 +2812,14 @@ gedit_window_create_tab_from_uri (GeditWindow         *window,
 
 	g_return_val_if_fail (GEDIT_IS_WINDOW (window), NULL);
 	g_return_val_if_fail (uri != NULL, NULL);
-	
+
 	tab = _gedit_tab_new_from_uri (uri,
 				       encoding,
 				       line_pos,
 				       create);	
 	if (tab == NULL)
 		return NULL;
-		
+
 	gtk_widget_show (tab);	
 	
 	gedit_notebook_add_tab (GEDIT_NOTEBOOK (window->priv->notebook),
@@ -2829,19 +2952,19 @@ _gedit_window_set_statusbar_visible (GeditWindow *window,
 				     gboolean     visible)
 {
 	GtkAction *action;
-	static gboolean recursione_guard = FALSE;
+	static gboolean recursion_guard = FALSE;
 
 	gedit_debug (DEBUG_WINDOW);
 
 	g_return_if_fail (GEDIT_IS_WINDOW (window));
 
-	if (recursione_guard)
+	if (recursion_guard)
 		return;
 
-	recursione_guard = TRUE;
+	recursion_guard = TRUE;
 
 	visible = (visible != FALSE);
-		
+
 	if (visible)
 		gtk_widget_show (window->priv->statusbar);
 	else
@@ -2849,14 +2972,14 @@ _gedit_window_set_statusbar_visible (GeditWindow *window,
 
 	if (gedit_prefs_manager_statusbar_visible_can_set ())
 		gedit_prefs_manager_set_statusbar_visible (visible);
-		
+
 	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
 					      "ViewStatusbar");		
-		
+
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
-	recursione_guard = FALSE;		
+	recursion_guard = FALSE;		
 }
 
 void
@@ -2864,17 +2987,17 @@ _gedit_window_set_toolbar_visible (GeditWindow *window,
 				   gboolean     visible)
 {
 	GtkAction *action;
-	static gboolean recursione_guard = FALSE;
+	static gboolean recursion_guard = FALSE;
 
 	gedit_debug (DEBUG_WINDOW);
 
 	g_return_if_fail (GEDIT_IS_WINDOW (window));
 
-	if (recursione_guard)
+	if (recursion_guard)
 		return;
 
-	recursione_guard = TRUE;
-	
+	recursion_guard = TRUE;
+
 	visible = (visible != FALSE);
 
 	if (visible)
@@ -2891,7 +3014,7 @@ _gedit_window_set_toolbar_visible (GeditWindow *window,
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), visible);
 
-	recursione_guard = FALSE;
+	recursion_guard = FALSE;
 }
 
 void
@@ -2981,7 +3104,7 @@ _gedit_window_set_bottom_panel_visible (GeditWindow *window,
 	if (gedit_prefs_manager_bottom_panel_visible_can_set ())
 		gedit_prefs_manager_set_bottom_panel_visible (visible);
 
-	action = gtk_action_group_get_action (window->priv->always_sensitive_action_group,
+	action = gtk_action_group_get_action (window->priv->action_group,
 					      "ViewBottomPanel");		
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) != visible)
@@ -3132,21 +3255,21 @@ gedit_window_get_unsaved_documents (GeditWindow *window)
 
 void 
 _gedit_window_set_saving_session_state (GeditWindow *window,
-				        gboolean     saving_session)
+					gboolean     saving_session)
 {
 	g_return_if_fail (GEDIT_IS_WINDOW (window));
-		
-	GeditWindowState old_state = window->priv->state;
 	
+	GeditWindowState old_state = window->priv->state;
+
 	if (saving_session)
 		window->priv->state |= GEDIT_WINDOW_STATE_SAVING_SESSION;
 	else
 		window->priv->state &= ~GEDIT_WINDOW_STATE_SAVING_SESSION;
-		
+
 	if (old_state != window->priv->state)
 	{
 		set_sensitivity_according_to_window_state (window);
-		
+
 		g_object_notify (G_OBJECT (window), "state");
 	}
 }
