@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit.c,v 1.128 2006/01/03 17:19:10 paolo Exp $
+ * $Id: gedit.c,v 1.130 2006/01/20 14:10:41 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -518,10 +518,14 @@ main (int argc, char *argv[])
 	gedit_debug_message (DEBUG_APP, "Start gtk-main");		
 	gtk_main();
 
+	bacon_message_connection_free (connection);
+
 	gedit_plugins_engine_shutdown ();
 	gedit_prefs_manager_app_shutdown ();
 	gedit_metadata_manager_shutdown ();
 
+	gnome_accelerators_sync ();
+
 	g_object_unref (program);
 
 	return 0;
